Type: Package
Package: chevreulShiny
Title: Tools for managing SingleCellExperiment objects as projects
Version: 1.5.0
Authors@R: c(
    person("Kevin", "Stachelek", , "kevin.stachelek@gmail.com", 
    role = c("aut", "cre"), comment = c(ORCID = "0000-0003-2085-695X")),
    person("Bhavana", "Bhat", , "bhavanabhat29@gmail.com", role = "aut")
    )
Description: Tools for managing SingleCellExperiment objects as projects. 
    Includes functions for analysis and visualization of single-cell data. 
    Also included is a shiny app for visualization of pre-processed scRNA data.
    Supported by NIH grants R01CA137124 and R01EY026661 to David Cobrinik.
License: MIT + file LICENSE
URL: https://github.com/whtns/chevreulShiny,
        https://whtns.github.io/chevreulShiny/
Date: 2024-03-24
BugReports: https://github.com/cobriniklab/chevreulShiny/issues
Depends: R (>= 4.5.0), SingleCellExperiment, shiny (>= 1.6.0),
        shinydashboard, chevreulProcess, chevreulPlot
Imports: alabaster.base, clustree, ComplexHeatmap, DataEditR (>=
        0.0.9), DBI, dplyr, DT, EnhancedVolcano, fs, future, ggplot2,
        ggplotify, grDevices, methods, patchwork, plotly, purrr,
        rappdirs, readr, RSQLite, S4Vectors, scales, shinyFiles,
        shinyhelper, shinyjs, shinyWidgets, stats, stringr, tibble,
        tidyr, tidyselect, utils, waiter, wiggleplotr
Suggests: BiocStyle, knitr, RefManageR, rmarkdown, testthat (>= 3.0.0),
        EnsDb.Mmusculus.v79, EnsDb.Hsapiens.v86
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: false
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
biocViews: Coverage, RNASeq, Sequencing, Visualization, GeneExpression,
        Transcription, SingleCell, Transcriptomics, Normalization,
        Preprocessing, QualityControl, DimensionReduction, DataImport
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/chevreulShiny
git_branch: devel
git_last_commit: fa7a9d5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:49:16 UTC; biocbuild
Author: Kevin Stachelek [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2085-695X>),
  Bhavana Bhat [aut]
Maintainer: Kevin Stachelek <kevin.stachelek@gmail.com>
