#' Human transcripts to genes
#'
#' Lookup table for converting Human (*Homo sapiens*)
#' Ensembl transcript IDs to gene IDs based on genome assembly
#' GRCH38 from Ensembl.
#'
#' @docType data
#' @keywords datasets
#'
#' @details
#' Variables:
#'
#' - enstxp
#' - ensgene
#'
#' @source \url{http://ensembl.org/homo_sapiens}
#'
#' @examples
#' data(grch38_tx2gene)
#' head(grch38_tx2gene)
"grch38_tx2gene"
