utils::globalVariables(c(".", "Cluster", "DefaultAssay", "FDR", "Gene.Name", "assay", "coverage", "group", "HGNC.symbol", "Idents", "MGI.symbol", "RunUMAP", "SID", "addPerCellQC", "algorithm", "bigWig", "cc.genes", "cc.genes.cyclone", "cluster", "colourInput", "colour_group", "condition", "dashboardPage", "db", "end.umap_1", "end.umap_2", "enriched_pathways_by_cluster_info", "enstxp", "feature", "format_pathway_table", "gene_index", "getEnrichedPathways", "gg_fill_hue group", "human_to_mouse_homologs", "meta_type", "name", "num_levels", "obect_pipeline", "object", "plotUMAP", "plot_ridge", "precedence", "proj_dir", "project_name", "project_path", "project_slug", "project_type", "rn", "sample_id", "start.umap_1", "start.umap_2", "summary.logFC", "symbol", "transcript", "updateButton", "write_csv", "x", "x_edit", "xend", "y", "yend", "DataFrame", "gg_fill_hue", "grch38", "grch38_tx2gene", "grcm38", "grcm38_tx2gene", "bigwig_db", "colData<-"))
