#' Human annotation data
#'
#' Human (*Homo sapiens*) annotations based on
#' genome assembly GRCH38 from Ensembl.
#'
#' @docType data
#' @keywords datasets
#'
#' @details
#' Variables:
#'
#' - ensgene
#' - entrez
#' - symbol
#' - chr
#' - start
#' - end
#' - strand
#' - biotype
#' - description
#'
#' @source \url{http://ensembl.org/homo_sapiens}
#'
#' @examples
#' data("grch38")
#' head(grch38)
"grch38"
