% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chevreulShiny-package.R
\docType{package}
\name{chevreulShiny-package}
\alias{chevreulShiny}
\alias{chevreulShiny-package}
\title{chevreulShiny: Tools for managing SingleCellExperiment objects as projects}
\description{
Tools for managing SingleCellExperiment objects as projects. Includes functions for analysis and visualization of single-cell data. Also included is a shiny app for visualization of pre-processed scRNA data. Supported by NIH grants R01CA137124 and R01EY026661 to David Cobrinik.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/whtns/chevreulShiny}
  \item \url{https://whtns.github.io/chevreulShiny/}
  \item Report bugs at \url{https://github.com/cobriniklab/chevreulShiny/issues}
}

}
\author{
\strong{Maintainer}: Kevin Stachelek \email{kevin.stachelek@gmail.com} (\href{https://orcid.org/0000-0003-2085-695X}{ORCID})

Authors:
\itemize{
  \item Bhavana Bhat \email{bhavanabhat29@gmail.com}
}

}
\keyword{internal}
