% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minimal_chevreul_app.R
\name{minimalChevreulApp}
\alias{minimalChevreulApp}
\title{Create a minimal chevreulShiny app using SingleCellExperiment input}
\usage{
minimalChevreulApp(
  single_cell_sce = NULL,
  appTitle = NULL,
  organism_type = "human",
  futureMb = 13000,
  db_name = "single-cell-projects.db"
)
}
\arguments{
\item{single_cell_sce}{a singlecell object}

\item{appTitle}{a title for the app}

\item{organism_type}{human or mouse}

\item{futureMb}{the megabytes available for the future package}

\item{db_name}{a database of bigwig files}
}
\value{
a minimal chevreulShiny app
}
\description{
Create a minimal chevreulShiny app using SingleCellExperiment input
}
\examples{

if (interactive() ) {
  data("tiny_sce")
  minimalChevreulApp(tiny_sce)
}

}
